package com.xunwang.education.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xunwang.commonutils.R;
import com.xunwang.commonutils.RedisIdWorker;
import com.xunwang.education.entity.StuNeeds;
import com.xunwang.education.entity.TeacherCertifications;
import com.xunwang.education.entity.TeacherCourses;
import com.xunwang.education.entity.vo.TeacherList;
import com.xunwang.education.entity.vo.TeacherViewCountVo;
import com.xunwang.education.feign.ImageFeign;
import com.xunwang.education.mapper.StudentRequirementsMapper;
import com.xunwang.education.mapper.TeacherCoursesMapper;
import com.xunwang.education.service.TeacherCertificationsService;
import com.xunwang.education.service.TeacherCoursesService;
import io.swagger.annotations.ApiOperation;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.*;

/**
 * @author Guan
 * @date 2023/1/11
 * discription
 */
@RestController
@RequestMapping("/education/home")
public class HomeController {

    @Resource
    private StudentRequirementsMapper studentRequirementsMapper;

    @Resource
    private TeacherCoursesMapper teacherCoursesMapper;

    @Resource
    private ImageFeign imageFeign;

    @Resource
    private StringRedisTemplate stringRedisTemplate;

    @GetMapping("/getHottestTeacher")
    public R getHottestTeacher(){
        String key = "viewCount:course";
        Set<String> strings = stringRedisTemplate.opsForZSet().reverseRange(key, 0, 2);
        List<TeacherList> teacherLists = null;
        if (strings != null) {
            teacherLists = teacherCoursesMapper.selectByBatch(new ArrayList<String>(strings));
        }

        // 利用feign:将头像地址转为base64字符串
        for (TeacherList teacherList : teacherLists) {
            String profilePhoto = teacherList.getProfilePhoto();
            String encode = imageFeign.download(profilePhoto);
            teacherList.setProfilePhoto(encode);
        }
        return R.ok().data("teacher", teacherLists);
    }

    @GetMapping("/getHottestStuNeed")
    public R getHottestStuNeed(){
        String key = "viewCount:stuNeed";
        Set<String> strings = stringRedisTemplate.opsForZSet().reverseRange(key, 0, 2);
        List<StuNeeds> stuNeedLists = null;
        if (strings != null) {
            stuNeedLists = studentRequirementsMapper.selectBatch(new ArrayList<String>(strings));
        }

        return R.ok().data("stuNeed", stuNeedLists);
    }



}
