package com.xunwang.education.controller;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.gson.Gson;
import com.xunwang.commonutils.JwtUtils;
import com.xunwang.commonutils.R;
import com.xunwang.commonutils.RedisIdWorker;
import com.xunwang.education.entity.*;
import com.xunwang.education.entity.socket.Commucation;
import com.xunwang.education.entity.socket.Type;
import com.xunwang.education.entity.vo.InviteCourseVo;
import com.xunwang.education.entity.vo.InviteStuNeedsVo;
import com.xunwang.education.feign.ImageFeign;
import com.xunwang.education.feign.PayFeign;
import com.xunwang.education.handler.ChatEndpoint;
import com.xunwang.education.service.InviteInfoService;
import com.xunwang.education.service.TeacherCertificationsService;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.List;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author Guan
 * @since 2023-01-03
 */
@RestController
@RequestMapping("/education/invite-info")
public class InviteInfoController {

    @Autowired
    private InviteInfoService inviteInfoService;

    @Autowired
    private RedisIdWorker redisIdWorker;

    @Resource
    private TeacherCertificationsService certificationsService;

    @Resource
    private ImageFeign imageFeign;

    @Resource
    private PayFeign payFeign;

    @Resource
    private TeacherCoursesController coursesController;

    @Resource
    private StudentRequirementsController requirementsController;

    @Resource
    private ChatEndpoint chatEndpoint;

    @ApiOperation("根据邀请号获取邀请信息")
    @GetMapping("/getInviteInfo")
    public R getInviteInfo(HttpServletRequest request) {
        String id = request.getParameter("id");
        InviteInfo one = inviteInfoService.getById(id);
        return R.ok().data("one", one);
    }

    @ApiOperation("找学员发送邀请")
    @PostMapping("/sendInvite")
    public R SendInvite(@RequestBody InviteCourseVo inviteCourseVo) throws IOException {
        LambdaQueryWrapper<InviteInfo> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(InviteInfo::getRequireId, inviteCourseVo.getRequireId()).eq(InviteInfo::getCourseId, inviteCourseVo.getCourseId()).eq(InviteInfo::getFromUid, inviteCourseVo.getUid()).eq(InviteInfo::getToUid, inviteCourseVo.getToUid());
        int count = inviteInfoService.count(wrapper);
        if (count > 0) {
            inviteInfoService.remove(wrapper);
        }

        InviteInfo inviteInfo = new InviteInfo();
        inviteInfo.setId(Long.toHexString(redisIdWorker.nextId("inviteId")));
        inviteInfo.setFromUname(inviteCourseVo.getFromUname());
        inviteInfo.setToUname(inviteCourseVo.getToUname());
        inviteInfo.setFromUid(inviteCourseVo.getUid());
        inviteInfo.setToUid(inviteCourseVo.getToUid());
        inviteInfo.setRequireId(inviteCourseVo.getRequireId());
        inviteInfo.setCourseId(inviteCourseVo.getCourseId());
        inviteInfo.setRequireSalary(new BigDecimal(inviteCourseVo.getSalary()));
        inviteInfo.setIdentity("教员");

        boolean save = inviteInfoService.save(inviteInfo);
        if (save) {
            // 5.向客户端(被邀请方)主动推送消息
            chatEndpoint.sendInvited(inviteInfo.getToUid(), inviteInfo.getId(), Commucation.INVITED.getMessage(), Type.INVITATION.getType());
            return R.ok().message("邀请成功");
        } else {
            return R.ok().message("邀请失败");
        }
    }

    @ApiOperation("找老师发送邀请")
    @PostMapping("/sendTeacherInvite")
    public R sendTeacherInvite(@RequestBody InviteStuNeedsVo inviteCourseVo) throws IOException {
        LambdaQueryWrapper<InviteInfo> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(InviteInfo::getRequireId, inviteCourseVo.getRequireId()).eq(InviteInfo::getCourseId, inviteCourseVo.getCourseId()).eq(InviteInfo::getFromUid, inviteCourseVo.getUid()).eq(InviteInfo::getToUid, inviteCourseVo.getToUid());
        int count = inviteInfoService.count(wrapper);
        if (count > 0) {
            inviteInfoService.remove(wrapper);
        }

        InviteInfo inviteInfo = new InviteInfo();
        inviteInfo.setId(Long.toHexString(redisIdWorker.nextId("inviteId")));
        inviteInfo.setFromUname(inviteCourseVo.getFromUname());
        inviteInfo.setToUname(inviteCourseVo.getToUname());
        inviteInfo.setFromUid(inviteCourseVo.getUid());
        inviteInfo.setToUid(inviteCourseVo.getToUid());
        inviteInfo.setRequireId(inviteCourseVo.getRequireId());
        inviteInfo.setCourseId(inviteCourseVo.getCourseId());
        inviteInfo.setRequireSalary(inviteCourseVo.getSalary());
        inviteInfo.setIdentity("学员");

        boolean save = inviteInfoService.save(inviteInfo);
        if (save) {
            // 5.向客户端(被邀请方)主动推送消息
            chatEndpoint.sendInvited(inviteInfo.getToUid(), inviteInfo.getId(), Commucation.INVITED.getMessage(), Type.INVITATION.getType());
            return R.ok().message("邀请成功");
        } else {
            return R.ok().message("邀请失败");
        }
    }

    @ApiOperation("获取已读的邀请list")
    @GetMapping("/getSendedInvitationRead")
    public R getSendedInvitation(HttpServletRequest request){
        String id = JwtUtils.getUIdByJwtToken(request);
        QueryWrapper<InviteInfo> wrapper = new QueryWrapper<>();
        wrapper.eq("from_uid",id).eq("read_status","已读").orderByDesc("datetime");
        List<InviteInfo> list = inviteInfoService.list(wrapper);
        return R.ok().data("list",list);
    }


    @ApiOperation("获取未读的邀请list")
    @GetMapping("/getSendedInvitationUnread")
    public R getSendedInvitationUnread(HttpServletRequest request){
        String id = JwtUtils.getUIdByJwtToken(request);
        QueryWrapper<InviteInfo> wrapper = new QueryWrapper<>();
        wrapper.eq("from_uid",id).eq("read_status","未读").orderByDesc("datetime");
        List<InviteInfo> list = inviteInfoService.list(wrapper);
        return R.ok().data("list",list);
    }

    @ApiOperation("我收到的邀请list已读")
    @GetMapping("/getReceivedInvitationRead")
    public R getReceivedInvitationRead(HttpServletRequest request){
        String id = JwtUtils.getUIdByJwtToken(request);
        QueryWrapper<InviteInfo> wrapper = new QueryWrapper<>();
        wrapper.eq("to_uid",id).eq("read_status","已读").orderByDesc("datetime");
        List<InviteInfo> list = inviteInfoService.list(wrapper);
        return R.ok().data("list",list);
    }

    @ApiOperation("我收到的邀请list未读")
    @GetMapping("/getReceivedInvitationUnread")
    public R getReceivedInvitationUnread(HttpServletRequest request){
        String id = JwtUtils.getUIdByJwtToken(request);
        QueryWrapper<InviteInfo> wrapper = new QueryWrapper<>();
        wrapper.eq("to_uid",id).eq("read_status","未读").orderByDesc("datetime");
        List<InviteInfo> list = inviteInfoService.list(wrapper);
        return R.ok().data("list",list);
    }


    //点击进入我收到的邀请详情，改变我方状态从未读变为待定，如果状态为非未读状态则不做修改
    @ApiOperation("修改我方操作状态为待定,阅读状态为已读")
    @GetMapping("/modifyStatusToHold")
    public R modifyStatusToHold(HttpServletRequest request){
        String id = request.getParameter("id");
        InviteInfo inviteInfo = new InviteInfo();
        inviteInfo.setReadStatus("已读");
        QueryWrapper<InviteInfo> wrapper = new QueryWrapper<>();
        wrapper.eq("id",id);
        //修改操作状态
        inviteInfoService.modifyStatusToHold(id);
        //修改阅读状态
        inviteInfoService.update(inviteInfo,wrapper);
        return R.ok();
    }

    @ApiOperation("拒绝邀请")
    @GetMapping("/refuse")
    public R refuse(HttpServletRequest request){
        String id = request.getParameter("id");
        InviteInfo inviteInfo = new InviteInfo();
        inviteInfo.setOperateStatus("拒绝");
        QueryWrapper<InviteInfo> wrapper = new QueryWrapper<>();
        wrapper.eq("id",id);
        inviteInfoService.update(inviteInfo,wrapper);
        return R.ok();
    }

    @ApiOperation("同意邀请")
    @GetMapping("/agree")
    public R agree(HttpServletRequest request) throws IOException {
        String id = request.getParameter("id");
        InviteInfo invite = inviteInfoService.getById(id);
        String lockId = invite.getCourseId() + invite.getRequireId();

        // 1.查询课程是否下架,是否上锁
        String s = coursesController.queryLockOrDisplay(invite.getCourseId());
        String[] ss = s.split(" ");
        if (!"课程ok".equals(ss[0])) {
            return R.ok().message(s);
        }
        String courseVersion = ss[1];

        // 2.查询需求是否下架,是否上锁
        String s1 = requirementsController.queryLockOrDisplay(invite.getRequireId());
        String[] s11 = s1.split(" ");
        if (!"需求ok".equals(s11[0])) {
            return R.ok().message(s1);
        }
        String requireVersion = s11[1];

        boolean b;
        boolean b1;
        // 3.锁定需求号和课程号
        synchronized (lockId) {
            b = coursesController.lockCourseId(invite.getCourseId(), courseVersion);
            b1 = requirementsController.lockCourseId(invite.getRequireId(), requireVersion);
        }

        if (b == false || b1 == false) {
            return R.ok().message("同意失败，请稍后在试");
        }

        // 修改邀请信息
        InviteInfo inviteInfo = new InviteInfo();
        inviteInfo.setOperateStatus("同意");
        QueryWrapper<InviteInfo> wrapper = new QueryWrapper<>();
        wrapper.eq("id",id);
        inviteInfoService.update(inviteInfo,wrapper);

        // 4.新增订单表
        R r = payFeign.generateOrder(id);
        if ("教员".equals(invite.getIdentity())) {
            Object info = r.getData().get("info");
            Gson gson = new Gson();
            String s2 = gson.toJson(info);
            OrderInfo orderInfo = gson.fromJson(s2, OrderInfo.class);

            // 5.向客户端(教员)主动推送订单消息
            chatEndpoint.sendInvited(invite.getFromUid(), orderInfo.getOrderId(), Commucation.PAYNOTICE.getMessage(), Type.PAYNOTICE.getType());
            r.data("identify", "教员");
        } else {
            r.data("identify", "学员");
        }
        return r;
    }

    @ApiOperation("获取教师头像")
    @GetMapping("/getProfile")
    public R getProfile(HttpServletRequest request) {
        String id = request.getParameter("id");
        TeacherCertifications one = certificationsService.getOne(new LambdaQueryWrapper<TeacherCertifications>().eq(TeacherCertifications::getUid, id));
        if (one == null) {
            return R.ok().data("ima", null);
        }
        String profilePhoto = one.getProfilePhoto();
        String img = imageFeign.download(profilePhoto);
        return R.ok().data("img", img);
    }
}

