package com.xunwang.file.controller;

import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import com.xunwang.commonutils.R;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.codec.binary.Base64;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.imageio.stream.FileImageInputStream;
import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

@ApiSupport(author = "Long")
@Api("文件相关")
@RestController
@RequestMapping("/file")
public class FileUpload {

    @Value("${define.windows.path}")
    private String windowsPath;

    @ApiOperation("上传文件接口")
    @PostMapping("upload")
    public R upload(@RequestParam("file") MultipartFile file) {
        // 根据日期划分目录
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        String formatDate = dateFormat.format(new Date());

        String newPath = windowsPath + "\\" + formatDate;
        // 如果目录不存在创建
        File uploadDir = new File(newPath);
        if (!uploadDir.exists()) {
            uploadDir.mkdir();
        }
        // 获取原始文件名
        String originalFilename = file.getOriginalFilename();
        // 获取原始文件后缀名
        String suffixName = originalFilename.substring(originalFilename.lastIndexOf("."));

        // 重新生成名字
        String fileName = UUID.randomUUID().toString() + suffixName;
        File localFile = new File(newPath + "\\" + fileName);
        try {
            // 将上传的文件保存到本地
            file.transferTo(localFile);
            Map<String, Object> hashMap = new HashMap<>();
            hashMap.put("path", formatDate);

            hashMap.put("fileName", fileName);
            return R.ok().data(hashMap);

        } catch (Exception e) {
            e.printStackTrace();
            return R.error().message("文件上传失败！");
        }

    }

    @ApiOperation("根据路径返回图片base64编码")
    @PostMapping("download")
    public String download(@RequestBody String filePath) {
        byte[] data = this.getStream(windowsPath + "\\" + filePath);

        // 将字节数组转为base64
        String encode = Base64.encodeBase64String(data).replaceAll("/[\\r\\n]/g", "");
        return encode;
    }

    @ApiOperation("根据路径返回图片base64编码")
    @PostMapping("download1")
    public String download1(@RequestBody String filePath) {
        byte[] data = this.getStream(filePath);

        // 将字节数组转为base64
        String encode = Base64.encodeBase64String(data).replaceAll("/[\\r\\n]/g", "");
        return encode;
    }

    @ApiOperation("根据路径返回图片的二进制")
    @PostMapping("downloadBinary")
    public byte[] downloadBinary(@RequestBody String filePath) {
        byte[] stream = this.getStream(windowsPath + "\\" + filePath);
        return stream;
    }

    @ApiOperation("get方法返回图片的二进制")
    @GetMapping("getDownloadBinary")
    public byte[] getDownloadBinary(HttpServletRequest request) {
        String filePath = request.getParameter("filePath");
        byte[] stream = this.getStream(windowsPath + "\\" + filePath);
        return stream;
    }

    @ApiOperation("根据路径批量返回图片的二进制")
    @GetMapping("downloadBinaryList")
    public R downloadBinaryList(HttpServletRequest request) {
        String diplomaPhoto = request.getParameter("diplomaPhoto");
        byte[] diplomaPhotoByte = this.downloadBinary(windowsPath + "\\" +diplomaPhoto);
        String idCardBack = request.getParameter("idCardBack");
        byte[] idCardBackByte = this.downloadBinary(windowsPath + "\\" + idCardBack);
        String idCardFront = request.getParameter("idCardFront");
        byte[] idCardFrontByte = this.downloadBinary(windowsPath + "\\" + idCardFront);
        String profilePhoto = request.getParameter("profilePhoto");
        byte[] profilePhotoByte = this.downloadBinary(windowsPath + "\\" + profilePhoto);
        HashMap<String, Object> hashMap = new HashMap<>();
        hashMap.put("diplomaPhotoByte", diplomaPhotoByte);
        hashMap.put("idCardBackByte", idCardBackByte);
        hashMap.put("idCardFrontByte", idCardFrontByte);
        hashMap.put("profilePhotoByte", profilePhotoByte);
        return R.ok().data("hash", hashMap);
    }

    @ApiOperation("根据路径批量返回图片base编码")
    @GetMapping("downloadList")
    public R downloadList(HttpServletRequest request) {
        String diplomaPhoto = request.getParameter("diplomaPhoto");
        String diplomaPhotoBase64 = this.download1(windowsPath + "\\" +diplomaPhoto);
        String idCardBack = request.getParameter("idCardBack");
        String idCardBackBase64 = this.download1(windowsPath + "\\" + idCardBack);
        String idCardFront = request.getParameter("idCardFront");
        String idCardFrontBase64 = this.download1(windowsPath + "\\" + idCardFront);
        String profilePhoto = request.getParameter("profilePhoto");
        String profilePhotoBase64 = this.download1(windowsPath + "\\" + profilePhoto);
        HashMap<String, String> hashMap = new HashMap<>();
        hashMap.put("diplomaPhotoBase64", diplomaPhotoBase64);
        hashMap.put("idCardBackBase64", idCardBackBase64);
        hashMap.put("idCardFrontBase64", idCardFrontBase64);
        hashMap.put("profilePhotoBase64", profilePhotoBase64);
        return R.ok().data("hash", hashMap);
    }

    private byte[] getStream(String filePath) {
        byte[] data = null;
        FileImageInputStream input = null;
        String encode = "";
        try {
            input = new FileImageInputStream(new File(filePath));
            ByteArrayOutputStream output = new ByteArrayOutputStream();
            byte[] buf = new byte[1024];
            int numBytesRead = 0;
            while ((numBytesRead = input.read(buf)) != -1) {
                output.write(buf, 0, numBytesRead);
            }
            data = output.toByteArray();
            output.close();
            input.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return data;

    }
}
